﻿#version 330 core

//infos given from vertex shader
in vec3 vertex_normal;
in vec3 vertex_worldPos;
in vec2 uv;
in vec4 position;
in vec4 lightSpacePos;

//uniforms
uniform vec3 viewPos;
uniform sampler2D MainTexture;	//texture of the model
uniform sampler2D depthTexture;	//depth texture for shadow mapping
uniform vec2 uvTiling = vec2(1);

// Ouput data
out vec4 color;

struct PointLight{
	vec3 pos;
	vec3 col;
	float intens;
	float range;
};

//TODO: set this value to the actual used lights in the scene
#define NUM_POINT_LIGHTS 2
uniform PointLight pointLights[NUM_POINT_LIGHTS];

float calcShadow()
{
	// get the light-space position in the range [-1,1]
	vec3 projCoords = lightSpacePos.xyz / lightSpacePos.w;

	// transform coordinates to the range [0,1] so we can have a texture lookup
	projCoords = projCoords * 0.5 + 0.5;

	float currentDepth = projCoords.z;
	float bias = 0.005;	//bias, so we do not have line pattern of shadow

	//sample surrounding texels of depth map for smoother edges on the shadow
	float shadow = 0;
	vec2 texelSize = 1.0 / textureSize(depthTexture, 0);
	for (int x = -2; x <= 2; x++)
	{
		for (int y = -2; y <= 2 ; y++)
		{
			float sampledDepth = texture(depthTexture, projCoords.xy + vec2(x, y) * texelSize).r;
			shadow += currentDepth - bias > sampledDepth ? 0.2 : 1;
		}
	}
	shadow /= 25.0;

	//so we don't have shadows outside the view frustum and far plane of the shadowmap
	if (projCoords.z > 1.0) shadow = 1.0;

	return shadow;
}

vec3 calcLight(vec3 texel, PointLight light)
{
	texel = (texel.x > 0 || texel.y > 0 || texel.z > 0) ? texel : vec3(1);
	vec3 objectColor = vec3(1);

	float dist = length(light.pos - vertex_worldPos);
	if (dist >= light.range) return objectColor * texel * 0.02;

	vec3 lightColor = light.col;
	float lightPow = light.intens;

	//ambient calculations
    float ambient = 0.02;

	//diffuse calculations
	vec3 normal = normalize(vertex_normal);
	vec3 lDir = normalize(light.pos - vertex_worldPos);
	float diffuse = max(dot(normal, lDir), 0) * clamp((1/dist), 0, 1) * lightPow;;	//actual diffuse calc

	//specular calculations
	float specularStrength = 0.3;
	vec3 vDir = normalize(viewPos - vertex_worldPos); //view direction user - vertex
	vec3 rDir = reflect(-lDir, normal);	//reflect light Direction on the vertex normal
	float specular = pow(max(dot(vDir, rDir), 0.0), 16);	//actual specular calc

	return (ambient + diffuse + specular) * lightColor * objectColor * texel;
}

vec3 applyFog(vec3 col)
{
	vec3 fogColor = vec3(0.2156863, 0.1735566, 0.172549);
	fogColor = vec3(0);
	float FogDensity = 0.1;

	//exponential fog
	float dist = length(viewPos - vertex_worldPos);
	float fogFac = clamp(1.0 /exp(pow(dist * FogDensity, 2)), 0.0, 1.0);

	return mix(fogColor, col, fogFac);
}

void main()
{
	//calculate a simple phong lighting model
	vec3 texel = texture(MainTexture, uv*uvTiling).rgb;
	texel = (texel.r!=0 || texel.g!=0 || texel.b!=0) ? texel : vec3(1);

	vec3 result = vec3(0);

	for(int i=0; i < NUM_POINT_LIGHTS; i++) result += calcLight(texel, pointLights[i]);

	float shadow = calcShadow();

	result = applyFog(result * shadow);

    color = vec4(result, 1.0);
}